@@ -1,24 +1,29 @@
 from .base import (
-    CaptionConverter, CaptionNode, Caption, CaptionList, CaptionSet)
+    CaptionConverter, CaptionNode, Caption, CaptionList, CaptionSet,
+)
 from .dfxp import DFXPWriter, DFXPReader
+from .microdvd import MicroDVDReader, MicroDVDWriter
 from .sami import SAMIReader, SAMIWriter
 from .srt import SRTReader, SRTWriter
 from .scc import SCCReader, SCCWriter
+from .scc.translator import translate_scc
 from .webvtt import WebVTTReader, WebVTTWriter
 from .exceptions import (
-    CaptionReadError, CaptionReadNoCaptions, CaptionReadSyntaxError)
+    CaptionReadError, CaptionReadNoCaptions, CaptionReadSyntaxError,
+)
 
 
 __all__ = [
-    'CaptionConverter', 'DFXPReader', 'DFXPWriter',
-    'SAMIReader', 'SAMIWriter', 'SRTReader', 'SRTWriter',
-    'SCCReader', 'SCCWriter', 'WebVTTReader', 'WebVTTWriter',
+    'CaptionConverter', 'DFXPReader', 'DFXPWriter', 'MicroDVDReader',
+    'MicroDVDWriter', 'SAMIReader', 'SAMIWriter', 'SRTReader', 'SRTWriter',
+    'SCCReader', 'SCCWriter', 'translate_scc', 'WebVTTReader', 'WebVTTWriter',
     'CaptionReadError', 'CaptionReadNoCaptions', 'CaptionReadSyntaxError',
     'detect_format', 'CaptionNode', 'Caption', 'CaptionList', 'CaptionSet'
 ]
 
 SUPPORTED_READERS = (
-    DFXPReader, WebVTTReader, SAMIReader, SRTReader, SCCReader)
+    DFXPReader, MicroDVDReader, WebVTTReader, SAMIReader, SRTReader, SCCReader,
+)
 
 
 def detect_format(caps):
