@@ -1,16 +1,21 @@
 import re
 from copy import deepcopy
 from xml.sax.saxutils import escape
+
 from bs4 import BeautifulSoup, NavigableString
 
 from ..base import (
     BaseReader, BaseWriter, CaptionSet, CaptionList, Caption, CaptionNode,
-    DEFAULT_LANGUAGE_CODE)
+    DEFAULT_LANGUAGE_CODE,
+)
 from ..exceptions import (
-    CaptionReadNoCaptions, CaptionReadSyntaxError, InvalidInputError)
+    CaptionReadNoCaptions, CaptionReadSyntaxError, InvalidInputError,
+    CaptionReadTimingError,
+)
 from ..geometry import (
     Point, Stretch, UnitEnum, Padding, VerticalAlignmentEnum,
-    HorizontalAlignmentEnum, Alignment, Layout)
+    HorizontalAlignmentEnum, Alignment, Layout,
+)
 from ..utils import is_leaf
 
 __all__ = [
@@ -45,9 +50,8 @@ DFXP_DEFAULT_REGION_ID = 'bottom'
 
 
 class DFXPReader(BaseReader):
-
     def __init__(self, *args, **kw):
-        super(DFXPReader, self).__init__(*args, **kw)
+        super().__init__(*args, **kw)
         self.read_invalid_positioning = (
             kw.get('read_invalid_positioning', False))
         self.nodes = []
@@ -59,7 +63,7 @@ class DFXPReader(BaseReader):
             return False
 
     def read(self, content):
-        if type(content) != str:
+        if not isinstance(content, str):
             raise InvalidInputError('The content is not a unicode string.')
 
         dfxp_document = self._get_dfxp_parser_class()(
@@ -68,9 +72,12 @@ class DFXPReader(BaseReader):
         caption_dict = {}
         style_dict = {}
 
+        default_language = dfxp_document.tt.attrs.get('xml:lang',
+                                                      DEFAULT_LANGUAGE_CODE)
+
         # Each div represents all the captions for a single language.
         for div in dfxp_document.find_all('div'):
-            lang = div.attrs.get('xml:lang', DEFAULT_LANGUAGE_CODE)
+            lang = div.attrs.get('xml:lang', default_language)
 
             caption_dict[lang] = self._translate_div(div)
 
@@ -80,8 +87,7 @@ class DFXPReader(BaseReader):
                 # Don't create document styles for those styles that are
                 # descendants of <region> tags. See link:
                 # http://www.w3.org/TR/ttaf1-dfxp/#styling-vocabulary-style
-                if 'region' not in [
-                        parent_.name for parent_ in style.parents]:
+                if 'region' not in [parent_.name for parent_ in style.parents]:
                     style_dict[id_] = self._translate_style(style)
 
         caption_set = CaptionSet(caption_dict, styles=style_dict)
@@ -93,13 +99,13 @@ class DFXPReader(BaseReader):
 
     @staticmethod
     def _get_dfxp_parser_class():
-        """Hook method for providing a custom DFXP parser
-        """
+        """Hook method for providing a custom DFXP parser"""
         return LayoutAwareDFXPParser
 
     def _translate_div(self, div):
         return CaptionList(
-            [self._translate_p_tag(p_tag) for p_tag in div.find_all('p')],
+            [self._translate_p_tag(p_tag)
+             for p_tag in div.find_all('p') if p_tag.get_text().strip()],
             div.layout_info
         )
 
@@ -116,11 +122,21 @@ class DFXPReader(BaseReader):
         return None
 
     def _find_times(self, p_tag):
-        start = self._translate_time(p_tag['begin'])
+        begin = p_tag.get('begin')
+        if not begin:
+            raise CaptionReadTimingError(
+                f'Missing begin time on line {p_tag}.')
 
-        try:
+        end = p_tag.get('end')
+        dur = p_tag.get('dur')
+        if not end and not dur:
+            raise CaptionReadTimingError(
+                f'Missing end time or duration on line {p_tag}.')
+
+        start = self._translate_time(begin)
+        if end:
             end = self._translate_time(p_tag['end'])
-        except KeyError:
+        else:
             dur = self._translate_time(p_tag['dur'])
             end = start + dur
 
@@ -129,17 +145,30 @@ class DFXPReader(BaseReader):
     def _translate_time(self, stamp):
         if stamp[-1].isdigit():
             timesplit = stamp.split(':')
+
+            if len(timesplit) < 3:
+                raise CaptionReadSyntaxError(
+                    'Begin and end time should follow the '
+                    'hour:minute:seconds.milliseconds or '
+                    'hour:minute:seconds:frames format. '
+                    'Milliseconds and frames are optional.'
+                    f'Please correct the following time {stamp}')
+
             if '.' not in timesplit[2]:
                 timesplit[2] += '.000'
+
             secsplit = timesplit[2].split('.')
             if len(timesplit) > 3:
-                secsplit.append((int(timesplit[3]) / 30) * 100)
+                secsplit.append(float(timesplit[3]) / 30 * 1000000)
             while len(secsplit[1]) < 3:
                 secsplit[1] += '0'
-            microseconds = (int(timesplit[0]) * 3600000000 +
-                            int(timesplit[1]) * 60000000 +
-                            int(secsplit[0]) * 1000000 +
-                            int(secsplit[1]) * 1000)
+            microseconds = (int(timesplit[0]) * 3600000000
+                            + int(timesplit[1]) * 60000000
+                            + int(secsplit[0]) * 1000000
+                            + int(secsplit[1]) * 1000)
+            if len(secsplit) > 2:
+                microseconds += int(secsplit[2])
+
             return microseconds
         else:
             # Must be offset-time
@@ -155,7 +184,8 @@ class DFXPReader(BaseReader):
             elif metric == "ms":
                 microseconds = value * 1000
             else:
-                raise InvalidInputError("Unsupported offset-time metric " + metric)
+                raise InvalidInputError("Unsupported offset-time metric "
+                                        + metric)
 
             return int(microseconds)
 
@@ -163,7 +193,7 @@ class DFXPReader(BaseReader):
         # convert text
         if isinstance(tag, NavigableString):
             # strips indentation whitespace only
-            pattern = re.compile("^(?:[\n\r]+\s*)?(.+)")
+            pattern = re.compile("^(?:[\n\r]+\\s*)?(.+)")
             result = pattern.search(tag)
             if result:
                 # Escaping/unescaping xml entities is the responsibility of the
@@ -239,7 +269,8 @@ class DFXPReader(BaseReader):
                 attrs['italics'] = True
             elif arg.lower() == "tts:fontweight" and dfxp_attrs[arg] == "bold":
                 attrs['bold'] = True
-            elif arg.lower() == "tts:textdecoration" and "underline" in dfxp_attrs[arg].strip().split(" "):
+            elif (arg.lower() == "tts:textdecoration"
+                  and "underline" in dfxp_attrs[arg].strip().split(" ")):
                 attrs['underline'] = True
             elif arg.lower() == "tts:textalign":
                 attrs['text-align'] = dfxp_attrs[arg]
@@ -259,7 +290,7 @@ class DFXPWriter(BaseWriter):
         self.p_style = False
         self.open_span = False
         self.region_creator = None
-        super(DFXPWriter, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def write(self, caption_set, force=''):
         """Converts a CaptionSet into an equivalent corresponding DFXP file
@@ -267,7 +298,7 @@ class DFXPWriter(BaseWriter):
         :type caption_set: pycaption.base.CaptionSet
         :param force: only use this language, if available in the caption_set
 
-        :rtype: unicode
+        :rtype: str
         """
         dfxp = BeautifulSoup(DFXP_BASE_MARKUP, 'lxml-xml')
         dfxp.find('tt')['xml:lang'] = "en"
@@ -275,6 +306,9 @@ class DFXPWriter(BaseWriter):
         langs = caption_set.get_languages()
         if force in langs:
             langs = [force]
+            dfxp.find('tt')['xml:lang'] = force
+        else:
+            dfxp.find('tt')['xml:lang'] = "en"
 
         caption_set = deepcopy(caption_set)
 
@@ -296,14 +330,15 @@ class DFXPWriter(BaseWriter):
             dfxp = self._recreate_styling_tag(
                 DFXP_DEFAULT_STYLE_ID, DFXP_DEFAULT_STYLE, dfxp)
 
-        self.region_creator = self._get_region_creator_class()(dfxp, caption_set)
+        self.region_creator = self._get_region_creator_class()(
+            dfxp, caption_set)
         self.region_creator.create_document_regions()
 
         body = dfxp.find('body')
 
         for lang in langs:
             div = dfxp.new_tag('div')
-            div['xml:lang'] = str(lang)
+            div['xml:lang'] = lang
             self._assign_positioning_data(div, lang, caption_set)
 
             for caption in caption_set.get_captions(lang):
@@ -324,8 +359,7 @@ class DFXPWriter(BaseWriter):
 
     @staticmethod
     def _get_region_creator_class():
-        """Hook method for providing a custom RegionCreator
-        """
+        """Hook method for providing a custom RegionCreator"""
         return RegionCreator
 
     def _assign_positioning_data(self, tag, lang, caption_set=None,
@@ -333,7 +367,7 @@ class DFXPWriter(BaseWriter):
         """Modifies the current tag, assigning it the 'region' attribute.
 
         :param tag: the BeautifulSoup tag to be modified
-        :type lang: unicode
+        :type lang: str
         :param lang: the caption language
         :type caption_set: CaptionSet
         :param caption_set: The CaptionSet parent
@@ -407,18 +441,17 @@ class DFXPWriter(BaseWriter):
 
             content_with_style = _recreate_style(node.content, dfxp)
             for style, value in list(content_with_style.items()):
-                styles += ' %s="%s"' % (style, value)
+                styles += f' {style}="{value}"'
             if node.layout_info:
                 region_id, region_attribs = (
                     self.region_creator.get_positioning_info(
                         lang, caption_set, caption, node
                     ))
-                styles += ' region="{region_id}"'.format(
-                    region_id=region_id)
+                styles += f' region="{region_id}"'
                 if self.write_inline_positioning:
                     styles += ' ' + ' '.join(
                         [
-                            '{key}="{val}"'.format(key=k_, val=v_)
+                            f'{k_}="{v_}"'
                             for k_, v_ in list(region_attribs.items())
                         ]
                     )
@@ -426,7 +459,7 @@ class DFXPWriter(BaseWriter):
             if styles:
                 if self.open_span:
                     line = line.rstrip() + '</span> '
-                line += '<span%s>' % styles
+                line += f'<span{styles}>'
                 self.open_span = True
 
         elif self.open_span:
@@ -440,7 +473,7 @@ class DFXPWriter(BaseWriter):
         Escapes XML 1.0 illegal or discouraged characters
         For details see:
             - http://www.w3.org/TR/2008/REC-xml-20081126/#dt-chardata
-        :type s: unicode
+        :type s: str
         :param s: The content of a text node
         """
         return escape(s)
@@ -499,7 +532,7 @@ class LayoutAwareDFXPParser(BeautifulSoup):
         # Work around for lack of '&apos;' support in html.parser
         markup = markup.replace("&apos;", "'")
 
-        super(LayoutAwareDFXPParser, self).__init__(
+        super().__init__(
             markup, features, builder, parse_only, from_encoding, **kwargs)
 
         self.read_invalid_positioning = read_invalid_positioning
@@ -534,8 +567,7 @@ class LayoutAwareDFXPParser(BeautifulSoup):
 
     @staticmethod
     def _get_region_from_ancestors(element):
-        """Try to get the region ID from the nearest ancestor that has it
-        """
+        """Try to get the region ID from the nearest ancestor that has it"""
         region_id = None
         parent = element.parent
         while parent:
@@ -603,7 +635,7 @@ class LayoutAwareDFXPParser(BeautifulSoup):
 
         :param region_id: the id of the region to which the element is
             associated
-        :type region_id: unicode
+        :type region_id: str
         :param element: BeautifulSoup Tag or NavigableString; this only comes
             into action (at the moment) if the
         :rtype: Layout
@@ -613,8 +645,7 @@ class LayoutAwareDFXPParser(BeautifulSoup):
         if region_id is not None:
             region_tag = self.find('region', {'xml:id': region_id})
 
-        region_scraper = (
-            self._get_layout_info_scraper_class()(self, region_tag))
+        region_scraper = self._get_layout_info_scraper_class()(self, region_tag)
 
         layout_info = region_scraper.scrape_positioning_info(
             element, self.read_invalid_positioning
@@ -629,18 +660,16 @@ class LayoutAwareDFXPParser(BeautifulSoup):
 
     @staticmethod
     def _get_layout_info_scraper_class():
-        """Hook method for getting an implementation of a LayoutInfoScraper.
-        """
+        """Hook method for getting an implementation of a LayoutInfoScraper."""
         return LayoutInfoScraper
 
     @staticmethod
     def _get_layout_class():
-        """Hook method for providing the Layout class to use
-        """
+        """Hook method for providing the Layout class to use"""
         return Layout
 
 
-class LayoutInfoScraper(object):
+class LayoutInfoScraper:
     """Encapsulates the methods for determining the layout information about
     an element (with the element's region playing an important role).
     """
@@ -706,9 +735,8 @@ class LayoutInfoScraper(object):
                 'style', {'xml:id': referenced_style_id}
             )
 
-            referenced_styles = (
-                cls._get_style_reference_chain(
-                    referenced_style, styling_section)
+            referenced_styles = cls._get_style_reference_chain(
+                referenced_style, styling_section
             )
         return nested_styles + referenced_styles
 
@@ -745,8 +773,7 @@ class LayoutInfoScraper(object):
             elif len(referenced_styles) > 1:
                 raise CaptionReadSyntaxError(
                     "Invalid caption file. "
-                    "More than 1 style with 'xml:id': {id}"
-                    .format(id=reference)
+                    f"More than 1 style with 'xml:id': {reference}"
                 )
 
         return result
@@ -817,7 +844,7 @@ class LayoutInfoScraper(object):
         """Look up the given attribute on the element, and all the styles
         referenced by it.
 
-        :type attribute_name: unicode
+        :type attribute_name: str
         :param element: BeautifulSoup Tag or NavigableString
         :param factory: a function, to apply to the xml attribute
         :param ignore: a list of values to ignore
@@ -847,16 +874,16 @@ class LayoutInfoScraper(object):
         parents and all their styles (and referenced styles).
 
         :param element: BeautifulSoup Tag or NavigableString
-        :type attribute_name: unicode
+        :type attribute_name: str
         :param attribute_name: the name of the attribute to resolve
-        :type attribute_name: unicode
+        :type attribute_name: str
         :param factory: callable to transform the xml attribute into something
         :param ignore: iterable of values to ignore (will return None if the
             xml attribute is in that list)
         :param ignorecase: if True, the attribute will be searched in lowercase
             too
         :type ignorecase: bool
-        :rtype: unicode
+        :rtype: str
         :raises CaptionSyntaxError:
         """
         value = None
@@ -912,7 +939,7 @@ class LayoutInfoScraper(object):
         return extent
 
 
-class RegionCreator(object):
+class RegionCreator:
     """Creates the DFXP regions, and knows how retrieve them, for assigning
     region IDs to every element
 
@@ -996,8 +1023,8 @@ class RegionCreator(object):
 
         for region_spec in unique_layouts:
             if (
-                    region_spec.origin or region_spec.extent or
-                    region_spec.padding or region_spec.alignment):
+                    region_spec.origin or region_spec.extent
+                    or region_spec.padding or region_spec.alignment):
 
                 new_region = dfxp.new_tag('region')
                 new_id = id_factory()
@@ -1032,9 +1059,9 @@ class RegionCreator(object):
     def _get_new_id(self, prefix='r'):
         """Return new, unique ids (use an internal counter).
 
-        :type prefix: unicode
+        :type prefix: str
         """
-        new_id = str((prefix or '') + str(self._id_seed))
+        new_id = f'{prefix}{self._id_seed}'
         self._id_seed += 1
         return new_id
 
@@ -1053,13 +1080,13 @@ class RegionCreator(object):
         <div> tags mean the caption is None and caption_node is None.
         <p> tags mean the caption_node is None
 
-        :type lang: unicode
+        :type lang: str
         :param lang: the language of the current caption element
         :type caption_set: CaptionSet
         :type caption: Caption
         :type caption_node: CaptionNode
         :rtype: tuple
-        :return: (unicode, dict)
+        :return: (str, dict)
         """
         # More intelligent people would have used an elem.parent.parent..parent
         # pattern, but pycaption is not yet structured for this. 3 params
@@ -1092,8 +1119,7 @@ class RegionCreator(object):
         return region_id, positioning_attributes
 
     def cleanup_regions(self):
-        """Remove the unused regions from the output file
-        """
+        """Remove the unused regions from the output file"""
         layout_tag = self._dfxp.find('layout')
         if not layout_tag:
             return
@@ -1144,8 +1170,8 @@ def _create_internal_alignment(text_align, display_align):
         "before", "center" and "after",
     with the default of "before". These refer to top/bottom positioning.
 
-    :type text_align: unicode
-    :type display_align: unicode
+    :type text_align: str
+    :type display_align: str
     :rtype: Alignment
     """
     if not (text_align or display_align):
@@ -1159,8 +1185,8 @@ def _create_external_horizontal_alignment(horizontal_component):
     """From an internal horizontal alignment value, create a value to be used
     in the dfxp output file.
 
-    :type horizontal_component: unicode
-    :rtype: unicode
+    :type horizontal_component: str
+    :rtype: str
     """
     result = None
 
@@ -1182,8 +1208,8 @@ def _create_external_vertical_alignment(vertical_component):
     """Given an alignment value used in the internal representation of the
     caption, return a value usable in the actual dfxp output file.
 
-    :type vertical_component: unicode
-    :rtype: unicode
+    :type vertical_component: str
+    :rtype: str
     """
     result = None
 
@@ -1299,8 +1325,8 @@ class _OrderedSet(list):
     """
     def add(self, p_object):
         if p_object not in self:
-            super(_OrderedSet, self).append(p_object)
+            super().append(p_object)
 
     def discard(self, value):
         if value in self:
-            super(_OrderedSet, self).remove(value)
+            super().remove(value)
