@@ -1,11 +1,9 @@
-
-
 class CaptionReadError(Exception):
     """
     Generic error raised when the reading of the caption file failed.
     """
     def __str__(self):
-        return "%s(%s)" % (self.__class__.__name__, self.args)
+        return f'{self.__class__.__name__}({self.args[0]})'
 
 
 class CaptionReadNoCaptions(CaptionReadError):
@@ -36,5 +34,4 @@ class RelativizationError(Exception):
 
 
 class InvalidInputError(RuntimeError):
-    """ Error raised when the input is invalid (i.e. a unicode string)
-    """
+    """Error raised when the input is invalid (i.e. a unicode string)"""
